from collections import namedtuple
from decimal import Decimal
from enum import Enum
from functools import partial
from typing import Any, Dict, List, Union
from uuid import uuid4

import motor.motor_asyncio
import pymongo
import pytz
from bson import CodecOptions, Decimal128, ObjectId
from bson.codec_options import TypeRegistry
from bson.json_util import dumps, loads
from gridfs import GridFS, GridFSBucket, GridOut
from jcramda import (
    _,
    attr,
    compose,
    curry,
    enum_name,
    first,
    getitem,
    has_attr,
    if_else,
    in_,
    is_a,
    locnow,
    not_,
    obj,
    popitem,
    when,
)
from pymongo.collection import Collection, ReturnDocument
from pymongo.database import Database
from pymongo.results import InsertOneResult, UpdateResult

from jcutil.core import get_running_loop


def fallback_encoder(value):
    return when(
        (is_a(Decimal), Decimal128),
        # (is_a(NDFrame), df_to_dict),
        (is_a(InsertOneResult), compose(obj("insertedId"), attr("inserted_id"))),
        (is_a(UpdateResult), compose(obj("upsertedId"), attr("upserted_id"))),
        (has_attr("get"), lambda o: o.get()),
        (has_attr("result"), lambda o: o.result()),
        else_=str,
    )(value)


# class NDFrameCodec(TypeCodec):
#     python_type = pd.Series
#     bson_type = dict
#
#     def transform_bson(self, value):
#         return pd.Series(value)
#
#     def transform_python(self, value):
#         return df_to_dict(value)


_type_registry = TypeRegistry(
    # type_codecs=[NDFrameCodec()],
    fallback_encoder=fallback_encoder
)


class UniqFileGridFSBucket(GridFSBucket):
    def _find_one(self, filename):
        return [*self.find({"filename": filename}).sort("uploadDate", -1).limit(1)]

    def _create_proxy(self, out: GridOut, opened=False):
        fid: ObjectId = getattr(out, "_id")
        self.delete(fid)
        create_method = (
            self.open_upload_stream_with_id
            if opened
            else self.upload_from_stream_with_id
        )
        return partial(create_method, fid)

    def open_save_file(self, filename, **kwargs):
        open_by_id = compose(
            lambda grid_out: self._create_proxy(grid_out, True)(filename, **kwargs),
            first,
        )
        return compose(
            if_else(
                not_, lambda _: self.open_upload_stream(filename, **kwargs), open_by_id
            ),
            self._find_one,
        )(filename)

    def save_file(self, filename, **kwargs):
        upload_by_id = compose(
            lambda grid_out: self._create_proxy(grid_out)(filename, **kwargs),
            first,
        )

        return compose(
            if_else(
                not_,
                lambda _: self.upload_from_stream(filename, **kwargs),
                upload_by_id,
            ),
            self._find_one,
        )(filename)


class AsyncUniqFileGridFSBucket:
    """异步版本的UniqFileGridFSBucket"""

    def __init__(self, db, bucket_name="fs"):
        self.bucket = motor.motor_asyncio.AsyncIOMotorGridFSBucket(
            db, bucket_name=bucket_name
        )

    async def _find_one(self, filename):
        cursor = (
            self.bucket.find({"filename": filename}).sort("uploadDate", -1).limit(1)
        )
        docs = []
        async for doc in cursor:
            docs.append(doc)
        return docs

    async def delete(self, file_id):
        await self.bucket.delete(file_id)

    async def open_upload_stream(self, filename, **kwargs):
        return await self.bucket.open_upload_stream(filename, **kwargs)

    async def open_upload_stream_with_id(self, file_id, filename, **kwargs):
        return await self.bucket.open_upload_stream_with_id(file_id, filename, **kwargs)

    async def upload_from_stream(self, filename, source, **kwargs):
        return await self.bucket.upload_from_stream(filename, source, **kwargs)

    async def upload_from_stream_with_id(self, file_id, filename, source, **kwargs):
        return await self.bucket.upload_from_stream_with_id(
            file_id, filename, source, **kwargs
        )

    async def _create_proxy(self, grid_out: Any, opened=False):
        fid: ObjectId = getattr(grid_out, "_id")
        await self.delete(fid)
        if opened:
            return partial(self.open_upload_stream_with_id, fid)
        else:
            return partial(self.upload_from_stream_with_id, fid)

    async def open_save_file(self, filename, **kwargs):
        docs = await self._find_one(filename)
        if not docs:
            return await self.open_upload_stream(filename, **kwargs)
        else:
            proxy = await self._create_proxy(docs[0], True)
            return proxy(filename, **kwargs)

    async def save_file(self, filename, **kwargs):
        """保存文件"""
        docs = await self._find_one(filename)
        if not docs:
            return self.upload_from_stream(filename, **kwargs)
        else:
            proxy = await self._create_proxy(docs[0])
            return proxy(filename, **kwargs)

    def __getattr__(self, name: str) -> Any:
        if name in self.bucket.__dict__:
            return getattr(self.bucket, name)
        raise AttributeError(f"AsyncUniqFileGridFSBucket has no attribute '{name}'")


class MongoClient:
    """MongoDB客户端，同时提供同步和异步接口"""

    def __init__(self, uri: str, alias: str = None):
        self.uri = uri
        self.alias = alias if alias is not None else uuid4().hex
        self.sync_client = pymongo.MongoClient(uri)

        # Use the same event loop for all async operations
        loop = get_running_loop()
        self.async_client = motor.motor_asyncio.AsyncIOMotorClient(uri, io_loop=loop)

        self.default_db_name = (
            self.sync_client.get_default_database().name
            if self.sync_client.get_default_database() is not None
            else None
        )

        # 配置编解码选项
        self.codec_options = CodecOptions(
            tz_aware=True,
            type_registry=_type_registry,
            tzinfo=pytz.timezone("Asia/Shanghai"),
        )

    def get_database(self, db_name=None) -> Database:
        """获取同步数据库对象"""
        if db_name is None:
            return self.sync_client.get_default_database()
        return self.sync_client.get_database(db_name)

    def get_async_database(self, db_name=None):
        """获取异步数据库对象"""
        # Ensure we have a valid async client
        self._ensure_valid_async_client()

        if db_name is None:
            if self.default_db_name:
                return self.async_client[self.default_db_name]
            return self.async_client.get_default_database()
        return self.async_client.get_database(db_name)

    def get_collection(
        self, collection_name: Union[str, Enum], db_name=None
    ) -> Collection:
        """获取同步集合对象"""
        db = self.get_database(db_name)
        collection_name = enum_name(collection_name)
        return db.get_collection(collection_name, codec_options=self.codec_options)

    def get_async_collection(self, collection_name: Union[str, Enum], db_name=None):
        """获取异步集合对象"""
        # Ensure we have a valid async client
        self._ensure_valid_async_client()

        db = self.get_async_database(db_name)
        collection_name = enum_name(collection_name)
        return db.get_collection(collection_name, codec_options=self.codec_options)

    def _ensure_valid_async_client(self):
        """确保异步客户端使用有效的事件循环"""
        try:
            # Try to get the loop used by the async client
            loop = self.async_client.get_io_loop()
            # Check if the loop is closed
            if loop.is_closed():
                # Create a new async client with a valid loop
                loop = get_running_loop()
                self.async_client = motor.motor_asyncio.AsyncIOMotorClient(
                    self.uri, io_loop=loop
                )
        except (RuntimeError, AttributeError):
            # If we can't get the loop or there's another issue,
            # create a new async client with a valid loop
            loop = get_running_loop()
            self.async_client = motor.motor_asyncio.AsyncIOMotorClient(
                self.uri, io_loop=loop
            )

    def get_fs(self, db_name=None) -> GridFS:
        """获取同步GridFS对象"""
        db = self.get_database(db_name)
        return GridFS(db)

    def get_fs_bucket(self, db_name=None, bucket_name="fs") -> UniqFileGridFSBucket:
        """获取同步GridFSBucket对象"""
        db = self.get_database(db_name)
        return UniqFileGridFSBucket(db, bucket_name)

    def get_async_fs_bucket(
        self, db_name=None, bucket_name="fs"
    ) -> AsyncUniqFileGridFSBucket:
        """获取异步GridFSBucket对象"""
        db = self.get_async_database(db_name)
        return AsyncUniqFileGridFSBucket(db, bucket_name)

    # 同步操作方法
    def find(
        self,
        collection: Union[Collection, str, Enum],
        query: dict,
        db_name=None,
        **kwargs,
    ):
        """同步查询多条记录"""
        if isinstance(collection, (str, Enum)):
            collection = self.get_collection(collection, db_name)
        return list(collection.find(query, **kwargs))

    def find_one(
        self,
        collection: Union[Collection, str, Enum],
        query: dict,
        db_name=None,
        **kwargs,
    ):
        """同步查询单条记录"""
        if isinstance(collection, (str, Enum)):
            collection = self.get_collection(collection, db_name)
        return collection.find_one(query, **kwargs)

    def find_by_id(
        self,
        collection: Union[Collection, str, Enum],
        _id: Union[str, ObjectId],
        db_name=None,
        **kwargs,
    ):
        """同步按ID查询"""
        if isinstance(collection, (str, Enum)):
            collection = self.get_collection(collection, db_name)
        query_id = _id if isinstance(_id, ObjectId) else ObjectId(_id)
        return collection.find_one({"_id": query_id}, **kwargs)

    def find_in_ids(
        self,
        collection: Union[Collection, str, Enum],
        ids: List[str],
        db_name=None,
        **kwargs,
    ):
        """同步按多个ID查询"""
        if isinstance(collection, (str, Enum)):
            collection = self.get_collection(collection, db_name)
        query = {"_id": {"$in": [ObjectId(n) for n in ids if n is not None]}}
        return list(collection.find(query, **kwargs))

    def find_page(
        self,
        collection: Union[Collection, str, Enum],
        query: dict,
        page_size=10,
        page_no=1,
        sort=None,
        db_name=None,
        **kwargs,
    ):
        """同步分页查询"""
        if isinstance(collection, (str, Enum)):
            collection = self.get_collection(collection, db_name)
        skip = page_size * (page_no - 1)
        if sort is None:
            sort = [("createdTime", pymongo.DESCENDING)]
        query.setdefault("logicDeleted", False)
        cursor = collection.find(query, **kwargs).sort(sort).skip(skip).limit(page_size)
        total = collection.count_documents(query)
        return {
            "content": list(cursor),
            "pageNo": page_no,
            "pageSize": page_size,
            "totalCount": total,
        }

    def save(self, collection: Union[Collection, str, Enum], data: dict, db_name=None):
        """同步保存数据（插入或更新）

        Args:
            collection: 集合名称或枚举
            data: 要保存的数据
            db_name: 数据库名称，默认使用默认数据库

        Returns:
            Dict: 包含保存后数据的字典，如果是新插入会包含_id
        """
        if isinstance(collection, (str, Enum)):
            collection = self.get_collection(collection, db_name)

        data_copy = data.copy()  # 创建副本以避免修改原始数据

        if "_id" in data_copy:
            # 从更新数据中提取_id
            doc_id = data_copy["_id"]

            # 从$set中移除_id字段，避免MongoDB报错
            set_data = data_copy.copy()
            if "_id" in set_data:
                set_data.pop("_id")

            # 构建更新操作
            update_ops = {
                "$set": set_data,
                "$currentDate": {"updateTime": True},
                "$inc": {"__v": 1},
            }

            # 更新现有文档
            result = collection.update_one({"_id": doc_id}, update_ops, upsert=True)

            if result.upserted_id:
                data_copy["_id"] = result.upserted_id

            # 获取更新后的完整文档以包含updateTime和__v
            return self.find_by_id(collection, data_copy["_id"])
        else:
            # 插入新文档
            data_copy.setdefault("createTime", locnow())
            data_copy.setdefault("__v", 0)
            data_copy.setdefault("updateTime", locnow())

            result = collection.insert_one(data_copy)
            if result.inserted_id:
                data_copy["_id"] = result.inserted_id

        return data_copy

    def replace(
        self,
        collection: Union[Collection, str, Enum],
        _id: Union[str, ObjectId],
        data: dict,
        db_name=None,
    ):
        """同步替换数据"""
        if isinstance(collection, (str, Enum)):
            collection = self.get_collection(collection, db_name)

        query_id = _id if isinstance(_id, ObjectId) else ObjectId(_id)
        old = collection.find_one({"_id": query_id})

        if "_id" in data:
            data.pop("_id")

        data.setdefault("__v", 0)
        data["__v"] += 1
        data["updateTime"] = locnow()

        result = collection.replace_one({"_id": query_id}, data, upsert=True)
        if result.modified_count or result.upserted_id:
            return self.find_by_id(collection, query_id), old

        return None, old

    def delete(
        self,
        collection: Union[Collection, str, Enum],
        _id: Union[str, ObjectId],
        db_name=None,
    ):
        """同步删除数据"""
        if isinstance(collection, (str, Enum)):
            collection = self.get_collection(collection, db_name)

        query_id = _id if isinstance(_id, ObjectId) else ObjectId(_id)
        result = collection.delete_one({"_id": query_id})
        return result.deleted_count

    # 异步操作方法
    async def async_find(
        self, collection: Union[str, Enum], query: dict, db_name=None, **kwargs
    ):
        """异步查询多条记录"""
        async_collection = self.get_async_collection(collection, db_name)
        cursor = async_collection.find(query, **kwargs)
        result = []
        async for doc in cursor:
            result.append(doc)
        return result

    async def async_find_one(
        self, collection: Union[str, Enum], query: dict, db_name=None, **kwargs
    ):
        """异步查询单条记录"""
        async_collection = self.get_async_collection(collection, db_name)
        return await async_collection.find_one(query, **kwargs)

    async def async_find_by_id(
        self,
        collection: Union[str, Enum],
        _id: Union[str, ObjectId],
        db_name=None,
        **kwargs,
    ):
        """异步按ID查询"""
        query_id = _id if isinstance(_id, ObjectId) else ObjectId(_id)
        return await self.async_find_one(
            collection, {"_id": query_id}, db_name, **kwargs
        )

    async def async_find_in_ids(
        self, collection: Union[str, Enum], ids: List[str], db_name=None, **kwargs
    ):
        """异步按多个ID查询"""
        query = {"_id": {"$in": [ObjectId(n) for n in ids if n is not None]}}
        return await self.async_find(collection, query, db_name, **kwargs)

    async def async_find_page(
        self,
        collection: Union[str, Enum],
        query: dict,
        page_size=10,
        page_no=1,
        sort=None,
        db_name=None,
        **kwargs,
    ):
        """异步分页查询"""
        async_collection = self.get_async_collection(collection, db_name)
        skip = page_size * (page_no - 1)
        if sort is None:
            sort = [("createdTime", pymongo.DESCENDING)]
        query.setdefault("logicDeleted", False)

        cursor = (
            async_collection.find(query, **kwargs)
            .sort(sort)
            .skip(skip)
            .limit(page_size)
        )
        docs = []
        async for doc in cursor:
            docs.append(doc)

        total = await async_collection.count_documents(query)
        return {
            "content": docs,
            "pageNo": page_no,
            "pageSize": page_size,
            "totalCount": total,
        }

    async def async_save(self, collection: Union[str, Enum], data: dict, db_name=None):
        """异步保存数据（插入或更新）

        Args:
            collection: 集合名称或枚举
            data: 要保存的数据
            db_name: 数据库名称，默认使用默认数据库

        Returns:
            Dict: 包含保存后数据的字典，如果是新插入会包含_id
        """
        async_collection = self.get_async_collection(collection, db_name)
        data_copy = data.copy()  # 创建副本以避免修改原始数据

        if "_id" in data_copy:
            # 从更新数据中提取_id
            doc_id = data_copy["_id"]

            # 从$set中移除_id字段，避免MongoDB报错
            set_data = data_copy.copy()
            if "_id" in set_data:
                set_data.pop("_id")

            # 构建更新操作
            update_ops = {
                "$set": set_data,
                "$currentDate": {"updateTime": True},
                "$inc": {"__v": 1},
            }

            # 更新现有文档
            result = await async_collection.update_one(
                {"_id": doc_id}, update_ops, upsert=True
            )

            # motor的UpdateResult直接提供upserted_id属性
            if result.upserted_id:
                data_copy["_id"] = result.upserted_id

            # 获取更新后的完整文档以包含updateTime和__v
            return await self.async_find_by_id(collection, data_copy["_id"], db_name)
        else:
            # 插入新文档
            data_copy.setdefault("createTime", locnow())
            data_copy.setdefault("__v", 0)
            data_copy.setdefault("updateTime", locnow())

            # motor的InsertOneResult直接提供inserted_id属性
            result = await async_collection.insert_one(data_copy)
            if result.inserted_id:
                data_copy["_id"] = result.inserted_id

        return data_copy

    async def async_replace(
        self,
        collection: Union[str, Enum],
        _id: Union[str, ObjectId],
        data: dict,
        db_name=None,
    ):
        """异步替换数据

        Args:
            collection: 集合名称或枚举
            _id: 文档ID
            data: 替换的新数据
            db_name: 数据库名称，默认使用默认数据库

        Returns:
            Tuple[Dict, Dict]: 包含新文档和旧文档的元组，如果操作失败返回(None, old)
        """
        async_collection = self.get_async_collection(collection, db_name)
        data_copy = data.copy()  # 创建副本以避免修改原始数据

        query_id = _id if isinstance(_id, ObjectId) else ObjectId(_id)
        old = await async_collection.find_one({"_id": query_id})

        if "_id" in data_copy:
            data_copy.pop("_id")

        data_copy.setdefault("__v", 0)
        data_copy["__v"] += 1
        data_copy["updateTime"] = locnow()

        try:
            result = await async_collection.replace_one(
                {"_id": query_id}, data_copy, upsert=True
            )

            if result.modified_count > 0 or result.upserted_id:
                # 获取更新后的完整文档
                new_doc = await self.async_find_by_id(collection, query_id, db_name)
                return new_doc, old

            # 如果没有修改文档（可能数据相同），但文档存在
            if old:
                return old, old

            return None, old
        except Exception as e:
            # 记录异常，但不抛出，保持与同步API一致的行为
            print(f"Error in async_replace: {e}")
            return None, old

    async def async_delete(
        self, collection: Union[str, Enum], _id: Union[str, ObjectId], db_name=None
    ):
        """异步删除数据

        Args:
            collection: 集合名称或枚举
            _id: 要删除的文档ID
            db_name: 数据库名称，默认使用默认数据库

        Returns:
            int: 被删除的文档数量
        """
        async_collection = self.get_async_collection(collection, db_name)

        query_id = _id if isinstance(_id, ObjectId) else ObjectId(_id)
        try:
            result = await async_collection.delete_one({"_id": query_id})
            return result.deleted_count
        except Exception as e:
            print(f"Error in async_delete: {e}")
            return 0

    def create_proxy(self, collection_name: str, db_name=None):
        """创建集合代理对象，提供简化的操作方法"""
        proxy_obj = namedtuple(
            f"{self.alias}_{collection_name}", "all,find,add,update,replace,delete"
        )

        def all(**kwargs):
            return self.find(collection_name, {}, db_name, **kwargs)

        def find(query: dict, **kwargs):
            return self.find(collection_name, query, db_name, **kwargs)

        def add(data):
            data.setdefault("createTime", locnow())
            return self.save(collection_name, data, db_name)

        def update(_id, data, **kwargs):
            current = self.find_by_id(collection_name, _id, db_name)
            if current:
                data_to_update = current.copy()
                data_to_update.update(data)
                data_to_update["_id"] = current["_id"]
                result = self.save(collection_name, data_to_update, db_name)
                return result
            return None

        def replace(_id, data, **kwargs):
            new_doc, old = self.replace(collection_name, _id, data, db_name)
            return new_doc

        def delete(_id):
            return self.delete(collection_name, _id, db_name)

        return proxy_obj(all, find, add, update, replace, delete)

    def create_async_proxy(self, collection_name: str, db_name=None):
        """创建异步集合代理对象，提供简化的异步操作方法"""
        proxy_obj = namedtuple(
            f"{self.alias}_{collection_name}_async",
            "all,find,add,update,replace,delete",
        )

        async def all(**kwargs):
            return await self.async_find(collection_name, {}, db_name, **kwargs)

        async def find(query: dict, **kwargs):
            return await self.async_find(collection_name, query, db_name, **kwargs)

        async def add(data):
            data.setdefault("createTime", locnow())
            return await self.async_save(collection_name, data, db_name)

        async def update(_id, data, **kwargs):
            """更新文档，保留原始数据的其他字段

            Args:
                _id: 文档ID
                data: 要更新的数据字段

            Returns:
                Dict: 更新后的完整文档
            """
            current = await self.async_find_by_id(collection_name, _id, db_name)
            if current:
                data_to_update = current.copy()
                data_to_update.update(data)
                data_to_update["_id"] = current["_id"]
                return await self.async_save(collection_name, data_to_update, db_name)
            return None

        async def replace(_id, data, **kwargs):
            """完全替换文档的内容

            Args:
                _id: 文档ID
                data: 新的完整文档内容

            Returns:
                Dict: 替换后的新文档
            """
            new_doc, old = await self.async_replace(collection_name, _id, data, db_name)
            return new_doc

        async def delete(_id):
            """删除文档

            Args:
                _id: 要删除的文档ID

            Returns:
                int: 删除的文档数量
            """
            return await self.async_delete(collection_name, _id, db_name)

        return proxy_obj(all, find, add, update, replace, delete)

    def drop_collection(self, collection_name: str, db_name=None):
        """删除集合"""
        collection = self.get_collection(collection_name, db_name)
        collection.drop()

    async def async_drop_collection(self, collection_name: str, db_name=None):
        """异步删除集合"""
        async_collection = self.get_async_collection(collection_name, db_name)
        await async_collection.drop()

    async def async_drop_database(self, db_name=None):
        """异步删除数据库"""
        async_db = self.get_async_database(db_name)
        await async_db.drop()

    def drop_database(self, db_name=None):
        """删除数据库"""
        db = self.get_database(db_name)
        db.drop()

    def close(self):
        """关闭客户端"""
        self.sync_client.close()
        self.async_client.close()

    def __str__(self):
        return f"MongoClient(uri='{self.uri}', alias='{self.alias}')"

    def __repr__(self):
        return self.__str__()


# 全局客户端实例管理
__clients: Dict[str, MongoClient] = dict()


def exists(name: str):
    """检查指定名称的客户端是否存在"""
    return name in __clients


def new_client(uri: str, alias: str = None) -> MongoClient:
    """创建新的MongoDB客户端实例"""
    db_name = alias if alias is not None else uuid4().hex
    client = MongoClient(uri, db_name)
    __clients[db_name] = client
    return client


def get_client(alias: str = None) -> MongoClient:
    """获取指定别名的客户端实例"""
    if alias is None:
        alias = first(list(__clients.keys()))
    return __clients[alias]


def load(conf: dict):
    """从配置加载多个MongoDB连接"""
    if conf and len(conf) > 0:
        for key in conf:
            new_client(conf[key], key)
            # print(f'mongodb: [{key}] connected')


def instances():
    """获取所有客户端实例名称"""
    return list(__clients.keys())


# 兼容旧API的辅助函数
def conn(key=None) -> MongoClient:
    """获取客户端连接（兼容旧API）"""
    return get_client(key)


def get_collection(tag, collection: Union[str, Enum], db_name=None):
    """获取集合对象（兼容旧API）"""
    client = get_client(tag)
    return client.get_collection(collection, db_name)


def fs_client(key=None) -> GridFS:
    """获取GridFS对象（兼容旧API）"""
    client = get_client(key)
    return client.get_fs()


def fs_bucket(db_name, bucket_name="fs") -> UniqFileGridFSBucket:
    """获取GridFSBucket对象（兼容旧API）"""
    client = get_client(db_name)
    return client.get_fs_bucket(bucket_name=bucket_name)


# 兼容旧API的数据操作函数
@curry
def find(collection, query):
    """查询函数（兼容旧API）"""
    assert is_a(Collection, collection)
    return tuple(collection.find(query))


@curry
def find_one(collection, query):
    """查询单条记录（兼容旧API）"""
    assert is_a(Collection, collection)
    return collection.find_one(query)


@curry
def find_by(collection, key, value):
    """按字段查询（兼容旧API）"""
    return collection.find(filter={key: value})


@curry
def save(collection, data):
    """保存数据（兼容旧API）"""
    data_copy = data.copy()  # 创建副本以避免修改原始数据

    if "_id" in data_copy:
        doc_id = data_copy["_id"]

        # 从$set中移除_id字段，避免MongoDB报错
        set_data = data_copy.copy()
        if "_id" in set_data:
            set_data.pop("_id")

        # 构建更新操作，包含currentDate和inc
        update_ops = {
            "$set": set_data,
            "$currentDate": {"updateTime": True},
            "$inc": {"__v": 1},
        }

        r = collection.update_one({"_id": doc_id}, update_ops, upsert=True)

        if hasattr(r, "upserted_id") and r.upserted_id:
            data_copy["_id"] = r.upserted_id

        # 获取完整更新后的文档
        updated = collection.find_one({"_id": doc_id if doc_id else data_copy["_id"]})
        if updated:
            return updated
    else:
        # 添加createTime、updateTime和初始版本字段
        data_copy.setdefault("createTime", locnow())
        data_copy.setdefault("updateTime", locnow())
        data_copy.setdefault("__v", 0)

        r = collection.insert_one(data_copy)
        if hasattr(r, "inserted_id"):
            data_copy["_id"] = r.inserted_id

    return data_copy


@curry
def replace_one(collection, data):
    """替换数据（兼容旧API）"""
    assert "_id" in data, "must include _id field in data."
    query = {"_id": data["_id"]}
    old = collection.find_one(query)
    collection.replace_one(query, data, upsert=True)
    return data, old


def find_page(collection, query, page_size=10, page_no=1, sort=None):
    """分页查询（兼容旧API）"""
    assert is_a(Collection, collection)
    skip = page_size * (page_no - 1)
    if sort is None:
        sort = [("createdTime", pymongo.DESCENDING)]
    query["logicDeleted"] = False
    cursor = collection.find(query).sort(sort).skip(skip).limit(page_size)
    total = collection.count_documents(query)
    return {
        "content": list(cursor),
        "pageNo": page_no,
        "pageSize": page_size,
        "totalCount": total,
    }


def mdb_proxy(tag, collection_name):
    """创建集合代理（兼容旧API）"""
    client = get_client(tag)
    return client.create_proxy(collection_name)


# 工具函数
to_json = dumps
from_json = loads


class BaseModel:
    """基础模型类"""

    def __init__(self, db_or_client, collection_name):
        if isinstance(db_or_client, MongoClient):
            self.client = db_or_client
            self.collection = db_or_client.get_collection(collection_name)
        else:
            self.client = None
            self.collection = db_or_client.get_collection(collection_name)

    def find_by_id(self, _id: Union[str, ObjectId]):
        query_id = _id if isinstance(_id, ObjectId) else ObjectId(_id)
        return self.collection.find_one(filter={"_id": query_id})

    def find_in_ids(self, ids: List[str], *args):
        query = {"_id": {"$in": [ObjectId(n) for n in ids if n is not None]}}
        return list(self.collection.find(query, *args))

    def _save(self, sepc_field: str = None, **kwargs):
        assert (
            "_id" in kwargs or not not_(sepc_field) and sepc_field in kwargs
        ), f"must had [_id] field or [{sepc_field}] field"
        query = if_else(
            in_(_, "_id"),
            compose(obj("_id"), popitem("_id")),
            compose(obj(sepc_field), getitem(sepc_field)),
        )
        return self.collection.find_one_and_update(
            filter=query(kwargs),
            update={"$set": kwargs},
            upsert=True,
            return_document=ReturnDocument.AFTER,
        )



# pandas tools
from jcramda import attr, compose, curry, flat_concat, is_a, partial

try:
    import numpy as np
    import pandas as pd

    col_value = curry(lambda col, df: compose(lambda s: s.iloc[0], attr(col))(df))

    df_dt = partial(pd.to_datetime, errors="ignore")

    df_to_json = curry(
        lambda df: df.to_json(orient="records", default_handler=str, force_ascii=False)
    )

    ser_to_json = curry(
        lambda ser: ser.to_json(
            default_handler=str,
            force_ascii=False,
        )
    )

    def df_to_dict(df):
        return (
            df.to_dict(orient="records")
            if isinstance(df, pd.DataFrame)
            else df.to_dict()
        )

    TYPE_REGS = (
        (is_a(pd.DataFrame), lambda df: flat_concat(df.to_dict(orient="records"))),
        (is_a(pd.Series), lambda ser: flat_concat(ser.to_dict())),
        (is_a(np.bool_), bool),
    )

    __all__ = (
        "col_value",
        "df_dt",
        "df_to_json",
        "ser_to_json",
        "df_to_dict",
        "TYPE_REGS",
    )
except ModuleNotFoundError:
    TYPE_REGS = ()
    __all__ = ("TYPE_REGS",)



import logging
import os
import socket
from functools import lru_cache
from pathlib import Path

from dotenv import load_dotenv


@lru_cache()
def local_ip():
    with socket.socket(socket.AF_INET, socket.SOCK_DGRAM) as s:
        s.connect(("180.76.76.76", 80))
        r = s.getsockname()[0]
    return r


def hostname():
    return socket.gethostname()


def str_bool(raw):
    if raw and raw.lower() in ("true", "t", "1"):
        return True
    return False


class Envars:
    def __init__(self):
        load_dotenv()
        self.SCHEMA = os.getenv("APP_SCHEMA", "public")
        self.APP_ENV = os.getenv("APP_ENV", "prod")
        self.APP_DEBUG = str_bool(os.getenv("APP_DEBUG"))
        self.APP_PORT = int(os.getenv("APP_PORT", 5000))
        self.CLIENT_ID = os.getenv("CLIENT_ID", hostname())
        self.CLIENT_IP = os.getenv("CLIENT_IP", local_ip())
        self.APP_NAME = os.getenv("APP_NAME", "app")
        self.IS_WORKER = str_bool(os.getenv("IS_WORKER"))
        self.CONFIG_PATH = "/".join(
            [
                os.getenv("CONSUL_KV_PATH", "config"),
                self.APP_NAME.lower(),
                self.APP_ENV.lower(),
            ]
        )
        self.CACHE_PATH = Path(os.getenv("CACHE_PATH", "/tmp")).resolve()
        self.LOG_LEVEL = os.getenv("LOG_LEVEL", "INFO")
        logging.basicConfig(level=self.LOG_LEVEL)


# 兼容旧版本
if "global_envars" not in globals():
    global_envars = Envars()
    # 兼容旧版本的全局变量
    SCHEMA = global_envars.SCHEMA
    APP_ENV = global_envars.APP_ENV
    APP_DEBUG = global_envars.APP_DEBUG
    APP_PORT = global_envars.APP_PORT
    CLIENT_ID = global_envars.CLIENT_ID
    CLIENT_IP = global_envars.CLIENT_IP
    APP_NAME = global_envars.APP_NAME
    IS_WORKER = global_envars.IS_WORKER
    CONFIG_PATH = global_envars.CONFIG_PATH
    CACHE_PATH = global_envars.CACHE_PATH
    LOG_LEVEL = global_envars.LOG_LEVEL


__all__ = (
    "Envars",
    "load_dotenv",
    "global_envars",
    "SCHEMA",
    "APP_ENV",
    "APP_DEBUG",
    "APP_PORT",
    "CLIENT_ID",
    "CLIENT_IP",
    "APP_NAME",
    "IS_WORKER",
    "CONFIG_PATH",
    "CACHE_PATH",
    "LOG_LEVEL",
)



import os
import platform
from datetime import timedelta
from sys import version_info

if version_info.major >= 3 and version_info.minor > 7:
    from typing import Protocol, runtime_checkable

    @runtime_checkable
    class Writable(Protocol):
        def write(self, *args, **kwargs):
            ...

        def close(self):
            ...

    @runtime_checkable
    class Readable(Protocol):
        def read(self, *args, **kwargs):
            ...

        def close(self):
            ...

else:

    class Writable(object):
        def write(self, *args, **kwargs):
            ...

        def close(self):
            ...

    class Readable(object):
        def read(self, *args, **kwargs):
            ...

        def close(self):
            ...


CACHE_DEFAULT_DIR = os.getenv("TEMP") if platform.system() == "Windows" else "/tmp"
DEFAULT_CACHE_TIME = timedelta(seconds=30)  # 30 seconds



import asyncio
import logging
import os

import pytest
import pytest_asyncio
import yaml

from jcutil.drivers.redis import Lock, SpinLock, load, new_client


@pytest_asyncio.fixture
async def setup_redis():
    """设置Redis测试连接"""
    import redis

    # 首先尝试同步连接测试Redis是否可用
    try:
        # 如果存在测试配置文件，则使用配置文件
        redis_uri = "redis://127.0.0.1:6379"
        if os.path.exists("tests/config.yaml"):
            with open("tests/config.yaml", "r") as f:
                conf = yaml.safe_load(f)
                if "redis" in conf and "test" in conf["redis"]:
                    redis_uri = conf["redis"]["test"]

        # 使用同步客户端测试连接
        client = redis.from_url(redis_uri)
        client.ping()  # 测试连接是否正常
        client.close()  # 关闭同步连接

        # Redis可用,创建异步连接
        if os.path.exists("tests/config.yaml"):
            with open("tests/config.yaml", "r") as f:
                conf = yaml.safe_load(f)
                if "redis" in conf:
                    # 处理load函数返回的协程
                    load_result = load(conf["redis"])
                    if asyncio.iscoroutine(load_result):
                        await load_result
                    return

        # 否则使用默认连接
        await new_client(redis_uri, "test")

    except Exception as e:
        logging.error("Failed to connect to Redis: %s", e)
        pytest.skip("Redis connection failed")


@pytest.mark.asyncio
async def test_redis_basic_operations(setup_redis):
    """测试Redis基本操作"""
    # 获取连接
    client = setup_redis()

    # 测试设置和获取值
    test_key = "test_key"
    test_value = "test_value"

    # 确保键不存在
    await client.delete(test_key)

    # 测试设置值
    assert await client.set(test_key, test_value) is True

    # 测试获取值
    result = await client.get(test_key)
    assert result.decode("utf-8") == test_value

    # 测试键存在
    assert await client.exists(test_key) == 1

    # 测试删除键
    assert await client.delete(test_key) == 1

    # 确认键已删除
    assert await client.exists(test_key) == 0


@pytest.mark.asyncio
async def test_redis_expire(setup_redis):
    """测试Redis过期时间设置"""
    client = setup_redis()
    test_key = "expire_key"

    # 设置带过期时间的键
    await client.set(test_key, "temporary", ex=2)

    # 键应该存在
    assert await client.exists(test_key) == 1

    # 等待过期
    await asyncio.sleep(3)

    # 键应该已过期
    assert await client.exists(test_key) == 0


@pytest.mark.asyncio
async def test_spin_lock(setup_redis):
    """测试SpinLock功能"""
    setup_redis()
    lock_key = "test_lock"

    # 测试自动获取和释放锁
    async with SpinLock("test", lock_key):
        # 在锁内执行操作
        await asyncio.sleep(1)
        logging.info("holding lock for 1 second")

    # 测试手动锁操作
    lock = Lock("test", lock_key)
    acquired = await lock.acquire()
    assert acquired is True
    assert lock.locked is True

    # 尝试获取已被锁定的锁
    lock2 = Lock("test", lock_key)
    acquired2 = await lock2.acquire(blocking=False)
    assert acquired2 is False

    # 释放锁
    await lock.release()
    assert lock.locked is False

    # 现在应该能获取锁了
    acquired2 = await lock2.acquire(blocking=False)
    assert acquired2 is True

    # 释放第二个锁
    await lock2.release()



from html import escape

from jcramda import when
from jcramda.base.comparison import is_a_str, is_a_tuple


def _escape(raw):
    if raw.startswith("{{") and raw.endswith("}}"):
        return raw.strip("{{").strip("}}")
    if is_a_str(raw):
        return f"'{escape(raw)}'"
    return str(raw)


_value_check = when(
    (is_a_str, lambda s: f" = {_escape(s)}"),
    (is_a_tuple, lambda tp: f"{tp[0]} {_escape(tp[1])}"),
    else_=str,
)


class Where:
    def __init__(self, raw):
        self.raw = raw

    def __str__(self):
        tn = getattr(type(self.raw), "__name__")
        method = getattr(self, f"from_{tn}")
        if callable(method):
            return method()
        else:
            return str(self.raw)

    def from_dict(self):
        """
        Example:
        >>> w = where({'name': 'someone', 'age': ('>', 32), 'birthday': '{{CURRENT_DATE}}'})
        >> print(w)
           name = 'someone' AND 'age' > 32 AND birthday = CURRENT_DATE
        """
        return " AND ".join(
            [f"{key} {_value_check(value)}" for key, value in self.raw.items()]
        )



import os

import pytest
import yaml
from dotenv import load_dotenv

from jcutil.drivers.mongo import MongoClient

load_dotenv()

# 测试MongoDB连接
# 首先尝试从配置文件加载
config_mongo_uri = None
if os.path.exists("tests/config.yaml"):
    try:
        with open("tests/config.yaml", "r") as f:
            config = yaml.safe_load(f)
            if "mongo" in config and "test" in config["mongo"]:
                config_mongo_uri = config["mongo"]["test"]
    except Exception as e:
        print(f"读取配置文件失败: {e}")

# 如果配置文件中没有，则从环境变量读取，如果环境变量中也没有，则使用默认连接
TEST_MONGO_URI = (
    os.getenv("MONGO_URI") or config_mongo_uri or "mongodb://localhost:27017/test"
)
TEST_COLLECTION = "test_collection"  # 使用特定的测试集合名称，避免冲突


@pytest.fixture(scope="module")
def client():
    """准备测试客户端"""
    try:
        client = MongoClient(TEST_MONGO_URI, "test_client")
        # 测试连接是否有效
        db = client.get_database()
        db.list_collection_names()  # 这会触发连接验证
        print(f"成功连接到MongoDB: {TEST_MONGO_URI}")
    except Exception as e:
        print(f"连接MongoDB失败: {e}")
        pytest.skip(f"无法连接到MongoDB: {e}")
        return None

    yield client

    try:
        # 清理测试数据
        db = client.get_database()
        if TEST_COLLECTION in db.list_collection_names():
            db[TEST_COLLECTION].drop()
        # 关闭连接以防内存泄漏
        client.sync_client.close()
    except Exception as e:
        print(f"清理测试数据失败：{e}")


def test_sync_api(client):
    """测试同步API"""
    # 插入测试数据
    data = {"name": "test_item", "value": 100}
    result = client.save(TEST_COLLECTION, data)
    assert "_id" in result
    assert result["name"] == "test_item"

    # 查询数据
    found = client.find_by_id(TEST_COLLECTION, result["_id"])
    assert found["name"] == "test_item"

    # 更新数据
    data["value"] = 200
    updated = client.save(TEST_COLLECTION, data)
    assert updated["value"] == 200

    # 查询所有数据
    all_items = client.find(TEST_COLLECTION, {})
    assert len(all_items) > 0

    # 删除数据
    delete_result = client.delete(TEST_COLLECTION, result["_id"])
    assert delete_result == 1


@pytest.mark.asyncio
async def test_async_api(client):
    """测试异步API"""
    # 插入测试数据
    data = {"name": "async_test_item", "value": 300}
    result = await client.async_save(TEST_COLLECTION, data)
    assert "_id" in result
    assert result["name"] == "async_test_item"

    # 查询数据
    found = await client.async_find_by_id(TEST_COLLECTION, result["_id"])
    assert found["name"] == "async_test_item"

    # 更新数据
    data["value"] = 400
    updated = await client.async_save(TEST_COLLECTION, data)
    assert updated["value"] == 400

    # 查询所有数据
    all_items = await client.async_find(TEST_COLLECTION, {})
    assert len(all_items) > 0

    # 删除数据
    delete_result = await client.async_delete(TEST_COLLECTION, result["_id"])
    assert delete_result == 1


@pytest.mark.asyncio
async def test_proxy(client):
    """测试代理功能"""
    # 测试同步代理
    proxy = client.create_proxy(TEST_COLLECTION)
    data = {"name": "proxy_test", "value": 500}
    added = proxy.add(data)
    assert added["name"] == "proxy_test"

    # 存储添加的数据ID，用于后续清理
    added_id = added["_id"]

    try:
        # 测试异步代理
        async_proxy = await client.create_async_proxy(TEST_COLLECTION)
        async_data = {"name": "async_proxy_test", "value": 600}
        async_added = await async_proxy.add(async_data)
        assert async_added["name"] == "async_proxy_test"

        # 存储异步添加的数据ID，用于后续清理
        async_added_id = async_added["_id"]

        # 查询并验证
        items = proxy.all()
        assert len(items) >= 2

        async_items = await async_proxy.all()
        assert len(async_items) >= 2

        # 清理异步测试数据
        await async_proxy.delete(async_added_id)
    finally:
        # 无论测试成功与否，都要清理同步测试数据
        proxy.delete(added_id)



#!/usr/bin/env python3
"""
密码哈希和加密工具模块
提供基于 passlib 的密码安全处理工具
"""

import base64
import os
import random
import string
from enum import Enum
from typing import Optional, Tuple, Union

# Import the secure_compare function from our existing crypto module

try:
    from passlib.context import CryptContext
    from passlib.exc import PasswordValueError
    from passlib.hash import argon2, bcrypt, pbkdf2_sha256, sha512_crypt

    PASSLIB_AVAILABLE = True
except ImportError:
    PASSLIB_AVAILABLE = False

# 默认密码哈希上下文，包含多种算法以支持历史密码和升级
DEFAULT_CONTEXT = (
    CryptContext(
        schemes=["argon2", "bcrypt", "pbkdf2_sha256", "sha512_crypt"],
        default="argon2",
        # 当验证旧模式的密码时自动升级到新模式
        deprecated=["sha512_crypt"],
        # argon2 配置
        argon2__rounds=4,
        argon2__memory_cost=65536,
        # bcrypt 配置
        bcrypt__rounds=12,
        # pbkdf2_sha256 配置
        pbkdf2_sha256__rounds=150000,
    )
    if PASSLIB_AVAILABLE
    else None
)


class HashScheme(Enum):
    """支持的哈希算法"""

    ARGON2 = "argon2"
    BCRYPT = "bcrypt"
    PBKDF2 = "pbkdf2_sha256"
    SHA512 = "sha512_crypt"


def _require_passlib():
    """Verify that passlib is installed"""
    if not PASSLIB_AVAILABLE:
        raise ImportError(
            "Password hashing features require the 'passlib' library. "
            "Please install it with 'pip install passlib bcrypt argon2-cffi'"
        )


def password_hash(
    password: str, scheme: Union[str, HashScheme] = None, **kwargs
) -> str:
    """使用选定的哈希算法创建密码哈希

    Args:
        password: 原始密码
        scheme: 使用的哈希算法，默认为 argon2
        **kwargs: 额外的参数，用于特定哈希算法的配置

    Returns:
        格式化的密码哈希字符串

    Raises:
        ImportError: 如果找不到 passlib 库
    """
    _require_passlib()

    # 处理空值
    if password is None:
        raise ValueError("密码不能为空")

    # 处理枚举值
    if isinstance(scheme, HashScheme):
        scheme = scheme.value

    # 如果指定了特定算法和轮数
    if scheme and kwargs:
        if scheme == HashScheme.ARGON2.value:
            # Default settings for Argon2 (if not provided in kwargs)
            time_cost = kwargs.get("time_cost", 3)
            memory_cost = kwargs.get("memory_cost", 65536)
            parallelism = kwargs.get("parallelism", 4)
            return argon2.using(
                time_cost=time_cost, memory_cost=memory_cost, parallelism=parallelism
            ).hash(password)
        elif scheme == HashScheme.BCRYPT.value:
            rounds = kwargs.get("rounds", 12)
            return bcrypt.using(rounds=rounds).hash(password)
        elif scheme == HashScheme.PBKDF2.value:
            rounds = kwargs.get("rounds", 150000)
            return pbkdf2_sha256.using(rounds=rounds).hash(password)
        elif scheme == HashScheme.SHA512.value:
            rounds = kwargs.get("rounds", 100000)
            return sha512_crypt.using(rounds=rounds).hash(password)
        else:
            raise ValueError(f"不支持的哈希算法: {scheme}")

    # 使用默认上下文
    if scheme:
        return DEFAULT_CONTEXT.handler(scheme).hash(password)
    else:
        return DEFAULT_CONTEXT.hash(password)


def password_verify(password: str, hash_str: str) -> Tuple[bool, Optional[str]]:
    """验证密码与哈希是否匹配，并在需要时返回升级后的哈希

    Args:
        password: 原始密码
        hash_str: 存储的密码哈希

    Returns:
        (是否匹配, 升级后的哈希或 None)

    Raises:
        ImportError: 如果找不到 passlib 库
    """
    _require_passlib()

    try:
        # Identify which hash scheme was used
        if hash_str.startswith("$argon2"):
            hasher = argon2
        elif hash_str.startswith("$2"):  # BCrypt
            hasher = bcrypt
        elif hash_str.startswith("$pbkdf2-sha256"):
            hasher = pbkdf2_sha256
        elif hash_str.startswith("$6$"):  # SHA512
            hasher = sha512_crypt
        else:
            raise ValueError(f"Unrecognized password hash format: {hash_str[:10]}...")

        # Verify the password
        is_valid = hasher.verify(password, hash_str)

        # Check if hash needs upgrading (returns new hash or None)
        new_hash = (
            hasher.hash(password)
            if is_valid and hasher.needs_update(hash_str)
            else None
        )

        # If still using an older scheme and password is valid, upgrade to Argon2
        if is_valid and not hash_str.startswith("$argon2") and not new_hash:
            new_hash = password_hash(password, HashScheme.ARGON2)

        return is_valid, new_hash

    except (ValueError, PasswordValueError) as e:
        # Handle any errors during verification
        print(f"Password verification error: {e}")
        return False, None


def generate_token(length: int = 32, url_safe: bool = True) -> str:
    """生成高熵随机令牌

    Args:
        length: 令牌字节长度
        url_safe: 是否生成 URL 安全的 base64 令牌

    Returns:
        随机令牌字符串
    """
    if length < 16:
        raise ValueError("Token length should be at least 16 bytes for security")

    token_bytes = os.urandom(length)

    if url_safe:
        return base64.urlsafe_b64encode(token_bytes).decode("utf-8").rstrip("=")
    else:
        return base64.b64encode(token_bytes).decode("utf-8")


def generate_password(length: int = 12, complexity: int = 4) -> str:
    """生成随机安全密码

    Args:
        length: 密码长度
        complexity: 复杂度级别 (1-4)，决定使用的字符集

    Returns:
        随机密码字符串
    """
    if length < 8:
        raise ValueError("Password length should be at least 8 characters")

    if complexity < 1 or complexity > 4:
        raise ValueError("Complexity must be between 1 and 4")

    char_sets = [
        string.ascii_lowercase,  # Level 1: lowercase
        string.digits,  # Level 2: add digits
        string.ascii_uppercase,  # Level 3: add uppercase
        "!@#$%^&*()-_=+[]{}|;:,.<>?/~",  # Level 4: add special
    ]

    # Add character sets based on complexity
    available_chars = ""
    for i in range(complexity):
        available_chars += char_sets[i]

    # Ensure at least one character from each set is included
    password = []
    for i in range(complexity):
        password.append(random.choice(char_sets[i]))

    # Fill the rest of the password with random characters
    for _ in range(length - complexity):
        password.append(random.choice(available_chars))

    # Shuffle the password to avoid predictable pattern
    random.shuffle(password)

    return "".join(password)


# 代码测试部分
if __name__ == "__main__":
    pwd = "my_secure_password"

    # 测试默认哈希
    hash1 = password_hash(pwd)
    print(f"默认哈希: {hash1}")

    # 测试 bcrypt
    hash2 = password_hash(pwd, HashScheme.BCRYPT)
    print(f"BCrypt哈希: {hash2}")

    # 测试验证
    is_valid, new_hash = password_verify(pwd, hash1)
    print(f"验证结果: {is_valid}, 升级后的哈希: {new_hash or '无需升级'}")

    # 测试令牌生成
    token = generate_token()
    print(f"随机令牌: {token}")

    # 测试密码生成
    random_pwd = generate_password(length=12, complexity=4)
    print(f"随机密码: {random_pwd}")



#!/usr/bin/env python3
"""
密码哈希功能演示
展示如何使用 jcutil 的密码哈希、验证和安全功能
"""

from jcutil.crypto_utils import (
    HashScheme,
    generate_password,
    generate_token,
    password_hash,
    password_verify,
    secure_compare,
)


def main():
    """演示密码哈希和验证功能"""
    print("= 密码哈希功能演示 =\n")

    # 1. 创建测试密码
    print("生成随机安全密码:")
    auto_password = generate_password(length=12, complexity=4)
    print(f"  自动生成的密码: {auto_password}\n")

    # 也可以使用用户输入的密码
    user_password = "MySecurePassword123!"
    print(f"  用户输入的密码: {user_password}\n")

    # 2. 使用不同算法创建密码哈希
    print("创建密码哈希:")
    # 默认哈希 (Argon2)
    default_hash = password_hash(user_password)
    print(f"  默认哈希 (Argon2): {default_hash}")

    # BCrypt 哈希
    bcrypt_hash = password_hash(user_password, HashScheme.BCRYPT)
    print(f"  BCrypt 哈希: {bcrypt_hash}")

    # PBKDF2 哈希
    pbkdf2_hash = password_hash(user_password, HashScheme.PBKDF2, rounds=160000)
    print(f"  PBKDF2 哈希 (160k轮): {pbkdf2_hash}")

    # SHA512 哈希 (已废弃但仍支持验证)
    sha512_hash = password_hash(user_password, HashScheme.SHA512)
    print(f"  SHA512 哈希 (已废弃): {sha512_hash}\n")

    # 3. 验证密码
    print("验证密码:")
    # 正确密码验证
    is_valid, upgraded_hash = password_verify(user_password, default_hash)
    print(f"  原始密码验证: {is_valid}")
    if upgraded_hash:
        print(f"  升级后的哈希: {upgraded_hash}")

    # 错误密码验证
    wrong_password = "WrongPassword123"
    is_valid, _ = password_verify(wrong_password, default_hash)
    print(f"  错误密码验证: {is_valid}\n")

    # 4. 安全令牌生成
    print("安全令牌生成:")
    # URL 安全的令牌
    url_safe_token = generate_token(length=24, url_safe=True)
    print(f"  URL安全令牌: {url_safe_token}")

    # 标准 Base64 令牌
    std_token = generate_token(length=24, url_safe=False)
    print(f"  标准Base64令牌: {std_token}\n")

    # 5. 安全字符串比较
    print("安全字符串比较 (防止时序攻击):")
    token_a = "a1b2c3d4e5f6"
    token_b = "a1b2c3d4e5f6"
    token_c = "x1y2z3d4e5f6"

    print(f"  Token A: {token_a}")
    print(f"  Token B: {token_b}")
    print(f"  Token C: {token_c}")

    print(f"  A == B: {secure_compare(token_a, token_b)}")
    print(f"  A == C: {secure_compare(token_a, token_c)}")


if __name__ == "__main__":
    try:
        main()
    except ImportError as e:
        print(f"错误: {e}")
        print("\n请安装所需的库: pip install passlib")



import os

import pytest
from dotenv import load_dotenv

from jcutil.consul import ConfigFormat, ConsulClient, KvProperty, fetch_key, list_keys

# 从.env文件加载Consul配置
load_dotenv()


@pytest.fixture
def consul_client():
    """创建Consul客户端实例"""
    try:
        client = ConsulClient()
        # 测试连接是否有效
        client.kv_get("test")
        return client
    except Exception as e:
        pytest.skip(f"无法连接到Consul: {e}")


class TestA:
    name = KvProperty("name")
    bar = KvProperty("foo", format=ConfigFormat.Yaml, cached=True)

    def desc(self):
        print("my name is:", self.name)


@pytest.fixture
def setup_test_data(consul_client):
    """设置测试数据"""
    # 准备测试数据
    consul_client.kv_put("properties/TestA/name", "FooBar")
    consul_client.kv_put(
        "properties/TestA/foo", "key: value\nlist:\n  - item1\n  - item2"
    )

    yield

    # 清理测试数据
    consul_client.kv_delete("properties/TestA/name")
    consul_client.kv_delete("properties/TestA/foo")


# 如果没有配置Consul，则跳过测试
skip_reason = "需要配置Consul服务才能运行此测试"
skip_test = os.getenv("CONSUL_HTTP_ADDR") == "127.0.0.1:8500" and not os.path.exists(
    "/usr/bin/consul"
)


@pytest.mark.skipif(skip_test, reason=skip_reason)
def test_kvp(setup_test_data):
    """测试KvProperty功能"""
    ta = TestA()
    ta.desc()
    assert ta.name == "FooBar"

    # 测试YAML格式和缓存功能
    assert isinstance(ta.bar, dict)
    assert ta.bar.get("key") == "value"
    assert isinstance(ta.bar.get("list"), list)
    assert len(ta.bar.get("list")) == 2


@pytest.mark.skipif(skip_test, reason=skip_reason)
def test_consul_client(consul_client):
    """测试ConsulClient基本功能"""
    # 测试键值操作
    test_key = "test/consul_client/key1"
    test_value = "test_value"

    try:
        # 设置键值
        result = consul_client.kv_put(test_key, test_value)
        assert result is True

        # 获取键值
        index, data = consul_client.kv_get(test_key)
        assert data["Value"].decode() == test_value

        # 列出键
        keys = list_keys("test/consul_client", client=consul_client)
        assert len(keys) > 0
        assert any(k["Key"] == test_key for k in keys)

        # 使用fetch_key获取值
        value = fetch_key(test_key, client=consul_client)
        assert value == test_value

        # 使用不同格式获取值
        json_key = "test/consul_client/json"
        json_value = '{"name": "test", "value": 123}'
        consul_client.kv_put(json_key, json_value)

        json_data = fetch_key(json_key, fmt=ConfigFormat.Json, client=consul_client)
        assert isinstance(json_data, dict)
        assert json_data["name"] == "test"
        assert json_data["value"] == 123

    finally:
        # 清理测试数据
        consul_client.kv_delete(test_key)
        consul_client.kv_delete("test/consul_client/json")



import asyncio
import logging
from importlib import import_module


def smart_load(conf):
    """智能加载配置"""
    for key in conf:
        try:
            m = import_module(f".{key}", package=__name__)
            if hasattr(m, "load"):
                r = m.load(conf[key])
                if asyncio.iscoroutine(r):
                    asyncio.get_running_loop().create_task(r)
        except ModuleNotFoundError as err:
            logging.debug("load %s failed: %s", key, err)


__all__ = ("smart_load",)



from enum import Enum
from typing import Any, Callable, Dict, List, NoReturn, Optional, Tuple

import consul  # 明确导入consul模块

class ConfigFormat(Enum):
    Text: Callable
    Number: Callable
    Int: Callable
    Float: Callable
    Json: Callable
    Yaml: Callable
    Hcl: Callable

def path_join(*args: str) -> str: ...
def fetch_key(
    key_path: str, fmt: Callable = None, client: Optional["ConsulClient"] = None
) -> Any: ...
def register_service(service_name: str, **kwargs) -> NoReturn: ...
def deregister(service_id) -> NoReturn: ...
def get_services(client: Optional["ConsulClient"] = None) -> Dict: ...
def find_service(
    query: str, by_id: bool = False, client: Optional["ConsulClient"] = None
) -> Dict: ...
def register_check(name: str, check: Dict, **kwargs) -> None: ...
def deregister_check(check_id: str) -> None: ...
def create_session(name: str = None, ttl: str = None, **kwargs) -> str: ...
def destroy_session(session_id: str) -> bool: ...
def renew_session(session_id: str) -> bool: ...
def acquire_lock(key: str, session_id: str, value: str = None) -> bool: ...
def release_lock(key: str, session_id: str) -> bool: ...
def list_keys(prefix: str, client: Optional["ConsulClient"] = None) -> List[Dict]: ...

class KvProperty:
    def __init__(
        self,
        key: str,
        /,
        prefix: str = None,
        namespace: str = None,
        format: callable = None,
        cached: bool = None,
    ) -> None: ...

class ConsulClient:
    def __init__(
        self,
        host: str = "127.0.0.1",
        port: int = 8500,
        token: str = None,
        scheme: str = "http",
        consistency: str = "default",
        dc: str = None,
        verify: bool = True,
    ) -> None: ...
    @property
    def client(self) -> consul.Consul: ...
    def kv_get(self, key: str, **kwargs) -> Any: ...
    def kv_put(self, key: str, value: str, **kwargs) -> bool: ...
    def kv_list(self, prefix: str, **kwargs) -> Tuple[int, List[Dict]]: ...
    def kv_delete(self, key: str, **kwargs) -> bool: ...
    def service_register(self, name: str, **kwargs) -> None: ...
    def service_deregister(self, service_id: str) -> None: ...
    def services(self) -> Dict: ...
    def check_register(self, name: str, check: Dict, **kwargs) -> None: ...
    def check_deregister(self, check_id: str) -> None: ...
    def checks(self) -> Dict: ...
    def session_create(self, name: str = None, **kwargs) -> str: ...
    def session_destroy(self, session_id: str) -> bool: ...
    def session_renew(self, session_id: str) -> bool: ...
    def lock_acquire(self, key: str, session_id: str, value: str = None) -> bool: ...
    def lock_release(self, key: str, session_id: str) -> bool: ...
